       program diffuse
       implicit none
   
!
! simulation parameters
!
       integer, parameter :: totpoints=2000
       real, parameter    :: xleft=-12., xright=+12.
       real, parameter    :: kappa=1.
       integer, parameter :: nsteps=100000
       integer, parameter :: plotsteps=50

!
! the calculated temperature, and the known correct
! solution from theory
!
       real, allocatable :: x(:)
       real, allocatable :: temperature(:,:)
       real, allocatable :: theory(:)

       integer :: old, new
       integer :: step
       integer :: i
       integer :: red, grey, white
       real :: time
       real :: dt, dx
       real :: error
       real :: dummy(2,2) = 0.

!
!  parameters of the original temperature distribution
!
       real, parameter :: ao=1., sigmao = 1.
       real a, sigma

!
! set parameters
!
       dx = (xright-xleft)/(totpoints-1)
       dt = dx**2 * kappa/10.
       old = 1                  
       new = 2

! 
! allocate data, including ghost cells: old and new timestep
! theory doesn't need ghost cells, but we include it for simplicity
!
       allocate(temperature(totpoints+2,2))
       allocate(theory(totpoints+2))
       allocate(x(totpoints+2))
!
!  setup initial conditions
!
       time = 0.
       do i=1, totpoints+2
           x(i) = xleft + (i-1)*dx
           temperature(i,old) = ao*exp(-(x(i))**2 / (2.*sigmao**2))
           theory(i)          = ao*exp(-(x(i))**2 / (2.*sigmao**2))
       enddo

       call pgbeg(0, "/xwindow", 1, 1)
       call pgask(0)
       call pgenv(xleft, xright, 0., 1.5*ao, 0, 0)
       call pglab('x', 'Temperature', 'Diffusion Test')
       red = 2
       call pgscr(red,1.,0.,0.)
       grey = 3
       call pgscr(grey,.2,.2,.2)
       white=4
       call pgscr(white,1.0,1.0,1.0);
 
       call pgsls(1)
       call pgslw(1)
       call pgsci(grey)
       call pgline(totpoints+2, x, theory)
       call pgsls(2)
       call pgslw(3)
       call pgsci(red)
       call pgline(totpoints+2, x, temperature(:,old))

!
!  evolve
!
       do step=1, nsteps
!
! boundary conditions: keep endpoint temperatures fixed.
!
           temperature(1,new) = temperature(1,old)
           temperature(totpoints+2,new) = temperature(totpoints+2,old)
!
! update solution
!
           do i=2,totpoints+1
               temperature(i,new) = temperature(i,old) + dt*kappa/(dx**2) *  &
                          (temperature(i+1,old) -                  &
                            2.*temperature(i,old) +                &
                            temperature(i-1,old) )
           enddo
           time = time + dt

           if (mod(step, plotsteps) == 0) then
               call pgbbuf()
               call pgeras()
               call pgsls(2)
               call pgslw(12)
               call pgsci(red)
               call pgline(totpoints+2, x, temperature(:,new))
            endif
    
! 
! update correct solution
!
           sigma = sqrt(2.*kappa*time + sigmao**2)
           a = ao*sigmao/sigma
           do i=1, totpoints+2
                theory(i) = a*exp(-(x(i))**2 / (2.*sigma**2))
           enddo

           if (mod(step, plotsteps) == 0) then
               call pgsls(1)
               call pgslw(6)
               call pgsci(white)
               call pgline(totpoints+2, x, theory)
               call pgebuf()
           endif
           error = sqrt(sum((theory(1:totpoints+1) - temperature(1:totpoints+1,new))**2))
           print *, 'Step = ', step, 'Time = ', time, ' Err = ', error

           old = new
           new = 3 - new
       enddo

       call pgend
       deallocate(temperature)
       deallocate(theory)
       deallocate(x)
       return
       end
