#include <stdio.h>
#include "boundary_conds.h"
#include "domain.h"

int apply_all_bcs(domain_t *d, int *bcs) {
	const int dirns[4] = {XLEFT_DIR, XRIGHT_DIR, YLEFT_DIR, YRIGHT_DIR};
	int i;

	for (i=0; i<4; i++){
		apply_bc(d, dirns[i], bcs[i]);
	}

	return 0;
}


int apply_bc(domain_t *d, int dirn, int bc) {
	
	if ((dirn != XLEFT_DIR) && (dirn != XRIGHT_DIR) && (dirn != YLEFT_DIR) && (dirn != YRIGHT_DIR)) {
		fprintf(stderr,"apply_bc: invalid direction %d\n", dirn);
		return -1;
	}

	switch (bc) {
		case BC_REFLECT: 
			apply_reflect_bc(d, dirn);
			break;

		case BC_PERIODIC: 
			apply_periodic_bc(d, dirn); 
			break;

		case BC_ZEROGRAD: 
			apply_zerograd_bc(d, dirn);
			break;

		default: fprintf(stderr,"apply_bc: invalid boundary condition %d\n", bc);
			 return -2;

	}
	

	return 0;
}

int apply_reflect_bc(domain_t *d, int dirn) {
	int i,j;
	int nx,   ny;
	int nxo, nyo;
	int nxt,  nyt;
	int nguard = d->Nguard;

	nx = d->Nx;
	ny = d->Ny;
	nxo= nx + nguard;
	nyo= ny + nguard;
	nxt= nx + 2*nguard;
	nyt= ny + 2*nguard;

	switch (dirn) {
		case XLEFT_DIR: 
			for (i=0; i<nguard; i++) {
				for (j=0; j<nyt; j++) {
					d->data[DENSVAR][i][j] =   d->data[DENSVAR][2*nguard-1-i][j];
					d->data[PRESVAR][i][j] =   d->data[PRESVAR][2*nguard-1-i][j];
					d->data[ENERVAR][i][j] =   d->data[ENERVAR][2*nguard-1-i][j];
					d->data[TEMPVAR][i][j] =   d->data[TEMPVAR][2*nguard-1-i][j];
					d->data[MOMXVAR][i][j] = -(d->data[MOMXVAR][2*nguard-1-i][j]);
					d->data[MOMYVAR][i][j] =   d->data[MOMYVAR][2*nguard-1-i][j];
				}
			}
			break;

		case XRIGHT_DIR:
			for (i=nxo; i<nxt; i++) {
				for (j=0; j<nyt; j++) {
					d->data[DENSVAR][i][j] =   d->data[DENSVAR][(2*nxo-1)-i][j];
					d->data[PRESVAR][i][j] =   d->data[PRESVAR][(2*nxo-1)-i][j];
					d->data[ENERVAR][i][j] =   d->data[ENERVAR][(2*nxo-1)-i][j];
					d->data[TEMPVAR][i][j] =   d->data[TEMPVAR][(2*nxo-1)-i][j];
					d->data[MOMXVAR][i][j] = -(d->data[MOMXVAR][(2*nxo-1)-i][j]);
					d->data[MOMYVAR][i][j] =   d->data[MOMYVAR][(2*nxo-1)-i][j];
				}
			}
			break;

		case YLEFT_DIR: 
			for (i=0; i<nxt; i++) {
				for (j=0; j<nguard; j++) {
					d->data[DENSVAR][i][j] =   d->data[DENSVAR][i][2*nguard-1-j];
					d->data[PRESVAR][i][j] =   d->data[PRESVAR][i][2*nguard-1-j];
					d->data[ENERVAR][i][j] =   d->data[ENERVAR][i][2*nguard-1-j];
					d->data[TEMPVAR][i][j] =   d->data[TEMPVAR][i][2*nguard-1-j];
					d->data[MOMXVAR][i][j] =   d->data[MOMXVAR][i][2*nguard-1-j];
					d->data[MOMYVAR][i][j] = -(d->data[MOMYVAR][i][2*nguard-1-j]);
				}
			}
			break;

		case YRIGHT_DIR: 
			for (i=0; i<nxt; i++) {
				for (j=nyo; j<nyt; j++) {
					d->data[DENSVAR][i][j] =   d->data[DENSVAR][i][(2*nyo-1)-j];
					d->data[PRESVAR][i][j] =   d->data[PRESVAR][i][(2*nyo-1)-j];
					d->data[ENERVAR][i][j] =   d->data[ENERVAR][i][(2*nyo-1)-j];
					d->data[TEMPVAR][i][j] =   d->data[TEMPVAR][i][(2*nyo-1)-j];
					d->data[MOMXVAR][i][j] =   d->data[MOMXVAR][i][(2*nyo-1)-j];
					d->data[MOMYVAR][i][j] = -(d->data[MOMYVAR][i][(2*nyo-1)-j]);
				}
			}
			break;
	}
	return 0;
}

int apply_periodic_bc(domain_t *d, int dirn) {
	int i,j;
	int nx,  ny;
	int nxo, nyo;
	int nxt, nyt;
	int nguard = d->Nguard;

	nx = d->Nx;
	ny = d->Ny;
	nxo= nx + d->Nguard;
	nyo= ny + d->Nguard;
	nxt= nx + 2*(d->Nguard);
	nyt= ny + 2*(d->Nguard);

	switch (dirn) {
		case XLEFT_DIR: 
			for (i=0; i<nguard; i++) {
				for (j=0; j<nyt; j++) {
					d->data[DENSVAR][i][j] = d->data[DENSVAR][i+nx][j];
					d->data[PRESVAR][i][j] = d->data[PRESVAR][i+nx][j];
					d->data[ENERVAR][i][j] = d->data[ENERVAR][i+nx][j];
					d->data[TEMPVAR][i][j] = d->data[TEMPVAR][i+nx][j];
					d->data[MOMXVAR][i][j] = d->data[MOMXVAR][i+nx][j];
					d->data[MOMYVAR][i][j] = d->data[MOMYVAR][i+nx][j];
				}
			}
			break;

		case XRIGHT_DIR:
			for (i=nxo; i<nxt; i++) {
				for (j=0; j<nyt; j++) {
					d->data[DENSVAR][i][j] = d->data[DENSVAR][i-nx][j];
					d->data[PRESVAR][i][j] = d->data[PRESVAR][i-nx][j];
					d->data[ENERVAR][i][j] = d->data[ENERVAR][i-nx][j];
					d->data[TEMPVAR][i][j] = d->data[TEMPVAR][i-nx][j];
					d->data[MOMXVAR][i][j] = d->data[MOMXVAR][i-nx][j];
					d->data[MOMYVAR][i][j] = d->data[MOMYVAR][i-nx][j];
				}
			}
			break;

		case YLEFT_DIR: 
			for (i=0; i<nxt; i++) {
				for (j=0; j<nguard; j++) {
					d->data[DENSVAR][i][j] = d->data[DENSVAR][i][j+ny];
					d->data[PRESVAR][i][j] = d->data[PRESVAR][i][j+ny];
					d->data[ENERVAR][i][j] = d->data[ENERVAR][i][j+ny];
					d->data[TEMPVAR][i][j] = d->data[TEMPVAR][i][j+ny];
					d->data[MOMXVAR][i][j] = d->data[MOMXVAR][i][j+ny];
					d->data[MOMYVAR][i][j] = d->data[MOMYVAR][i][j+ny];
				}
			}
			break;

		case YRIGHT_DIR: 
			for (i=0; i<nxt; i++) {
				for (j=nyo; j<nyt; j++) {
					d->data[DENSVAR][i][j] = d->data[DENSVAR][i][j-ny];
					d->data[PRESVAR][i][j] = d->data[PRESVAR][i][j-ny];
					d->data[ENERVAR][i][j] = d->data[ENERVAR][i][j-ny];
					d->data[TEMPVAR][i][j] = d->data[TEMPVAR][i][j-ny];
					d->data[MOMXVAR][i][j] = d->data[MOMXVAR][i][j-ny];
					d->data[MOMYVAR][i][j] = d->data[MOMYVAR][i][j-ny] ;
				}
			}
			break;
	}
	return 0;
}

int apply_zerograd_bc(domain_t *d, int dirn) {
	int i,j;
	int nx,  ny;
	int nxo,nyo;
	int nxt, nyt;
	int nguard = d->Nguard;

	nx = d->Nx;
	ny = d->Ny;
	nxo= nx + d->Nguard;
	nyo= ny + d->Nguard;
	nxt= nx + 2*(d->Nguard);
	nyt= ny + 2*(d->Nguard);

	switch (dirn) {
		case XLEFT_DIR: 
			for (i=0; i<nguard; i++) {
				for (j=0; j<nyt; j++) {
					d->data[DENSVAR][i][j] = d->data[DENSVAR][nguard+1][j];
					d->data[PRESVAR][i][j] = d->data[PRESVAR][nguard+1][j];
					d->data[ENERVAR][i][j] = d->data[ENERVAR][nguard+1][j];
					d->data[TEMPVAR][i][j] = d->data[TEMPVAR][nguard+1][j];
					d->data[MOMXVAR][i][j] = d->data[MOMXVAR][nguard+1][j];
					d->data[MOMYVAR][i][j] = d->data[MOMYVAR][nguard+1][j];
				}
			}
			break;

		case XRIGHT_DIR:
			for (i=nxo; i<nxt; i++) {
				for (j=0; j<nyt; j++) {
					d->data[DENSVAR][i][j] = d->data[DENSVAR][nxo-1][j];
					d->data[PRESVAR][i][j] = d->data[PRESVAR][nxo-1][j];
					d->data[ENERVAR][i][j] = d->data[ENERVAR][nxo-1][j];
					d->data[TEMPVAR][i][j] = d->data[TEMPVAR][nxo-1][j];
					d->data[MOMXVAR][i][j] = d->data[MOMXVAR][nxo-1][j];
					d->data[MOMYVAR][i][j] = d->data[MOMYVAR][nxo-1][j];
				}
			}
			break;

		case YLEFT_DIR: 
			for (i=0; i<nxt; i++) {
				for (j=0; j<nguard; j++) {
					d->data[DENSVAR][i][j] = d->data[DENSVAR][i][nguard+1];
					d->data[PRESVAR][i][j] = d->data[PRESVAR][i][nguard+1];
					d->data[ENERVAR][i][j] = d->data[ENERVAR][i][nguard+1];
					d->data[TEMPVAR][i][j] = d->data[TEMPVAR][i][nguard+1];
					d->data[MOMXVAR][i][j] = d->data[MOMXVAR][i][nguard+1];
					d->data[MOMYVAR][i][j] = d->data[MOMYVAR][i][nguard+1];
				}
			}
			break;

		case YRIGHT_DIR: 
			for (i=0; i<nxt; i++) {
				for (j=nyo; j<nyt; j++) {
					d->data[DENSVAR][i][j] = d->data[DENSVAR][i][nyo-1];
					d->data[PRESVAR][i][j] = d->data[PRESVAR][i][nyo-1];
					d->data[ENERVAR][i][j] = d->data[ENERVAR][i][nyo-1];
					d->data[TEMPVAR][i][j] = d->data[TEMPVAR][i][nyo-1];
					d->data[MOMXVAR][i][j] = d->data[MOMXVAR][i][nyo-1];
					d->data[MOMYVAR][i][j] = d->data[MOMYVAR][i][nyo-1] ;
				}
			}
			break;
	}
	return 0;
}
