#include <getopt.h>
#include <stdio.h>
#include <stdlib.h>
#include "boundary_conds.h"
#include "init_conds.h"
int get_options(int argc, char **argv, int *npts, int *nsteps, int *xbc, int *ybc, int *simulation, int *outevery,
                                        double *dt, double *pener, double *pvel, double *psize, int *output, int *nxproc, int *nyproc) {

    char c;
    int option_index;
    int tempint;
    double tempflt;
    int myoutput=*output;

    struct option long_options[] = {
        {"output"   , no_argument, &myoutput, 1},
        {"nooutput" , no_argument, &myoutput, 0},
        {"npts"     , required_argument, 0, 'n'},
        {"nsteps"   , required_argument, 0, 't'},
        {"dt"       , required_argument, 0, 'd'},
        {"psize"    , required_argument, 0, 'r'},
        {"pvel"     , required_argument, 0, 'v'},
        {"pener"    , required_argument, 0, 'e'},
        {"xboundary", required_argument, 0, 'x'},
        {"yboundary", required_argument, 0, 'y'},
        {"outevery" , required_argument, 0, 'o'},
        {"simulation", required_argument, 0, 's'},
        {"nxproc", required_argument, 0, 'X'},
        {"nyproc", required_argument, 0, 'Y'},
        {"help",      no_argument, 0, 'h'},
        {0, 0, 0, 0}};


        *psize = 0.1;
        *pvel = 4.;
        *pener=0.851072;  /* a standard test case */

        while (1) {
                c = getopt_long(argc, argv, "n:t:d:r:v:x:y:o:h:e:s:X:Y", long_options, &option_index);
                if (c == -1) break;

                switch (c) {
                        case 0: if (long_options[option_index].flag != 0)
                                break;

                        case 'n': tempint = atoi(optarg);
                                  if (tempint < 1 || tempint > 1000) {
                                        fprintf(stderr,"%s: Cannot use number of points %s;\n  Using %d\n", argv[0], optarg, *npts);
                                  } else {
                                        *npts = tempint;
                                  }
                                  break;


                        case 't': tempint = atoi(optarg);
                                  if (tempint < 1 || tempint > 1000) {
                                        fprintf(stderr,"%s: Cannot use number of steps  %s;\n  Using %d\n", argv[0], optarg, *nsteps);
                                  } else {
                                        *nsteps  = tempint;
                                  }
                                  break;


                        case 'd': tempflt = atof(optarg);
                                  if (tempflt < 0. || tempflt > 1.) {
                                        fprintf(stderr,"%s: Cannot use timestep %s;\n  Using %g\n", argv[0], optarg, *dt);
                                  } else {
                                        *dt = tempflt;
                                  }
                                  break;


                        case 'e': tempflt = atof(optarg);
                                  if (tempflt < 0. || tempflt > 1.e10) {
                                        fprintf(stderr,"%s: Cannot use explosion energy %s;\n  Using %g\n", argv[0], optarg, *pener);
                                  } else {
                                        *pener = tempflt;
                                  }
                                  break;

                        case 'v': tempflt = atof(optarg);
                                  if (tempflt < 0. || tempflt > 10.) {
                                        fprintf(stderr,"%s: Cannot use velocity %s;\n  Using %g\n", argv[0], optarg, *pvel);
                                  } else {
                                        *pvel = tempflt;
                                  }
                                  break;

                        case 'r': tempflt = atof(optarg);
                                  if (tempflt < 0. || tempflt > 10.) {
                                        fprintf(stderr,"%s: Cannot use size %s;\n  Using %g\n", argv[0], optarg, *psize);
                                  } else {
                                        *psize = tempflt;
                                  }
                                  break;

                        case 'x': tempint = atoi(optarg);
                                  if (tempint < 1 || tempint > NBCS) {
                                        fprintf(stderr,"%s: invalid x boundary condition %s;\n  Using %d\n", argv[0], optarg, abs(*xbc));
                                  } else {
                                        *xbc = -tempint;
                                  }
                                  break;

                        case 'y': tempint = atoi(optarg);
                                  if (tempint < 1 || tempint > NBCS) {
                                        fprintf(stderr,"%s: invalid y boundary condition %s;\n  Using %d\n", argv[0], optarg, abs(*ybc));
                                  } else {
                                        *ybc = -tempint;
                                  }
                                  break;

                        case 'o': tempint = atoi(optarg);
                                  if (tempint < 1) {
                                        fprintf(stderr,"%s: invalid output frequency %s;\n  Using %d\n", argv[0], optarg, *outevery);
                                  } else {
                                        *outevery = tempint;
                                  }
                                  break;

                        case 's': tempint = atoi(optarg);
                                  if (tempint != SIM_PROJECTILE && tempint != SIM_SEDOV ) {
                                        fprintf(stderr,"%s: invalid simulation number %s;\n  Using %d\n", argv[0], optarg, *simulation);
                                  } else {
                                        *simulation = tempint;
                                  }
                                  break;
                        case 'X': tempint = atoi(optarg);
                                  if (tempint < 1 || tempint > 1000) {
                                        fprintf(stderr,"%s: invalid number of processors in X direction %s;\n  Using %d\n", argv[0], optarg, *nxproc);
                                  } else {
                                        *nxproc = tempint;
                                  }
                                  break;


                        case 'Y': tempint = atoi(optarg);
                                  if (tempint < 1 || tempint > 1000) {
                                        fprintf(stderr,"%s: invalid number of processors in Y direction %s;\n  Using %d\n", argv[0], optarg, *nyproc);
                                  } else {
                                        *nyproc = tempint;
                                  }
                                  break;


            case 'h': 
                  puts("Options: ");
                  puts("    --npts=N      (-n N): Set the number of grid cells in each direction.");
                  puts("    --nsteps=N    (-t N): Set the number of time steps taken.");
                  puts("    --dt=X        (-d X): Set the time step.");
                  puts("    --pvel=X      (-v X): Set the velocity parameter (projectile run).");
                  puts("    --pener=X     (-v X): Set the explosion parameter (Sedov run).");
                  puts("    --psize=X     (-r X): Set the size parameter.");
                  puts("    --simulation=N(-s N): Set the simulation; 0 == Sedov, 1 == projectile.");
                  puts("    --outevery=N  (-o N): Set output every N timesteps.");
                  puts("    --xboundary=N (-x N): Set the x-boundary condition: ");
                  puts("    --yboundary=N (-y N): Set the y-boundary condition: ");
                  puts("                          1=outflow, 2=reflecting, 3=periodic");
                  puts("    --nxproc=N    (-X N): Set the number of processors in the X direction.");
                  puts("    --nyproc=N    (-Y N): Set the number of processors in the Y direction.");
                  puts("");
                  return +1;

            default: printf("Invalid option %s\n", optarg);
                 break;
        }
    }
    *output = myoutput;
    return 0;
}

