#include <stdio.h>
#include <stdlib.h>
#include <omp.h>
#include <assert.h>
#include <math.h>
#include "../util/pca_utils.h"

/*--------------------------------------------------------------------------------*/
void print_mat(NType **mat, int n)
{
  for (int i=0;i<n;i++)
    {
      for (int j=0;j<n;j++)
	fprintf(stdout," %14.5e",mat[i][j]);
      fprintf(stdout,"\n");
    }
}

/*--------------------------------------------------------------------------------*/


void fill_random_matrix(NType **mat, int n)
{
  for (int i=0;i<n;i++)
    for (int j=0;j<n;j++)
      mat[i][j]=((NType)rand())/RAND_MAX;
}

/*--------------------------------------------------------------------------------*/
void matmult_block(NType **a, NType **b, NType **c, int n, int bs)
{

  for (int i=0;i<n;i++)
    for (int j=0;j<n;j++)
      c[i][j]=0;
  
#pragma omp parallel shared(a,b,c,n,bs) default(none)
 {
  
   int nb=n/bs;
   assert(nb*bs==n);  /*fail if we don't have an exact multiple of the block size*/
   
   
   
   NType **myblock_1=matrix(bs,bs);
   NType **myblock_2=matrix(bs,bs);
   NType **myblock_3=matrix(bs,bs);
   
#pragma omp for schedule(dynamic,1)
   for (int ib=0;ib<nb;ib++)
     for (int jb=0;jb<nb;jb++)       
       for (int kb=0;kb<nb;kb++) {
	 int ii=ib*bs;
	 int jj=jb*bs;
	 int kk=kb*bs;
	 //  Pull the blocks from A and B out
	 for (int i=0;i<bs;i++)
	   for (int j=0;j<bs;j++) {	    
	     myblock_1[i][j]=a[i+ii][j+kk];
	     myblock_2[i][j]=b[j+kk][i+jj];
	   }
	 //Do the block multiplication
	 for (int i=0;i<bs;i++)
	   for (int j=0;j<bs;j++) {	    
	     myblock_3[i][j]=0;
	       for (int k=0;k<bs;k++)
		 myblock_3[i][j]+=myblock_1[i][k]*myblock_2[j][k];
	   }
	 //Accumulate the product into c
	 for (int i=0;i<bs;i++)
	   for (int j=0;j<bs;j++)
	     c[i+ii][j+jj]+=myblock_3[i][j];
       }
   
   free_matrix(myblock_1);
   free_matrix(myblock_2);
   free_matrix(myblock_3);
 }
}



/*--------------------------------------------------------------------------------*/
NType matrix_sum(NType **a, int n)
{
  NType sum=0;
  for (int i=0;i<n;i++)
    for (int j=0;j<n;j++)
      sum+=a[i][j];
  return sum;
}

/*================================================================================*/

int main(int argc, char *argv[])
{
  pca_time tt;
  int n=1000;
  int bs=25;

  NType **a=matrix(n,n);
  NType **b=matrix(n,n);
  NType **c=matrix(n,n);

  fill_random_matrix(a,n);
  fill_random_matrix(b,n);

  tick(&tt);
  matmult_block(a,b,c,n,bs);
  printf("Time to multiply %d x %d matrices with block multiplication is %9.4f\n",n,n,tocksilent(&tt));
  printf("Sum of elements is %14.4e\n",matrix_sum(c,n));

}
