#include <stdio.h>
#include <stdlib.h>
#include <omp.h>
#include <assert.h>
#include <math.h>
#include "../util/pca_utils.h"


void dgemm_(char *transa, char *transb, int *m, int *n, int *k, double *alpha, double *a, int *lda, double *b, int *ldb,double *begta, double *c, int *ldc, int *info);



/*--------------------------------------------------------------------------------*/
void print_mat(NType **mat, int n)
{
  for (int i=0;i<n;i++)
    {
      for (int j=0;j<n;j++)
	fprintf(stdout," %14.5e",mat[i][j]);
      fprintf(stdout,"\n");
    }
}

/*--------------------------------------------------------------------------------*/


void fill_random_matrix(NType **mat, int n)
{
  for (int i=0;i<n;i++)
    for (int j=0;j<n;j++)
      mat[i][j]=((NType)rand())/RAND_MAX;
}
/*--------------------------------------------------------------------------------*/

void matmult_slowest(NType **a, NType **b, NType **c, int n)
{
  for (int i=0;i<n;i++)
    for (int j=0;j<n;j++)
      {
	c[i][j]=0;
	for (int k=0;k<n;k++)
	  c[i][j]+=a[i][k]*b[k][j];
      }
}
/*--------------------------------------------------------------------------------*/
NType matrix_sum(NType **a, int n)
{
  NType sum=0;
  for (int i=0;i<n;i++)
    for (int j=0;j<n;j++)
      sum+=a[i][j];
  return sum;
}

/*================================================================================*/

int main(int argc, char *argv[])
{
  pca_time tt;
  int n=1000;

  NType **a=matrix(n,n);
  NType **b=matrix(n,n);
  NType **c=matrix(n,n);

  fill_random_matrix(a,n);
  fill_random_matrix(b,n);

  tick(&tt);
  //matmult_slowest(a,b,c,n);
  double one=1.0;
  double zero=0.0;
  int info;
  dgemm_("N","N",&n,&n,&n,&one,a[0],&n,b[0],&n,&zero,c[0],&n,&info);

  printf("Time to multiply %d x %d matrices with blas multiplication is %9.4f\n",n,n,tocksilent(&tt));
  printf("Sum of elements is %14.4e\n",matrix_sum(c,n));

}
