#include <stdio.h>
#include <stdlib.h>
#include <omp.h>
#include <assert.h>
#include <math.h>
#include "../util/pca_utils.h"

/*--------------------------------------------------------------------------------*/
void print_mat(NType **mat, int n)
{
  for (int i=0;i<n;i++)
    {
      for (int j=0;j<n;j++)
	fprintf(stdout," %14.5e",mat[i][j]);
      fprintf(stdout,"\n");
    }
}

/*--------------------------------------------------------------------------------*/


void fill_random_matrix(NType **mat, int n)
{
  for (int i=0;i<n;i++)
    for (int j=0;j<n;j++)
      mat[i][j]=((NType)rand())/RAND_MAX;
}
/*--------------------------------------------------------------------------------*/
void matrix_transpose(NType **in, NType **out, int n)
{
  for (int i=0;i<n;i++)
    for (int j=0;j<n;j++)
      out[j][i]=in[i][j];
}

/*--------------------------------------------------------------------------------*/

void matmult_transpose(NType **a, NType **b, NType **c, int n)
{
  NType **bt=matrix(n,n);
  matrix_transpose(b,bt,n);


  for (int i=0;i<n;i++)
    for (int j=0;j<n;j++)  {
      c[i][j]=0;
      NType mysum=0;
#pragma omp parallel for shared(i,j,a,bt,n) reduction(+:mysum) default(none)      
      for (int k=0;k<n;k++)
	mysum+=a[i][k]*bt[j][k];
      c[i][j]=mysum;
    }
  
  free_matrix(bt);
}
/*--------------------------------------------------------------------------------*/
NType matrix_sum(NType **a, int n)
{
  NType sum=0;
  for (int i=0;i<n;i++)
    for (int j=0;j<n;j++)
      sum+=a[i][j];
  return sum;
}

/*================================================================================*/

int main(int argc, char *argv[])
{
  pca_time tt;
  int n=1000;

  NType **a=matrix(n,n);
  NType **b=matrix(n,n);
  NType **c=matrix(n,n);

  fill_random_matrix(a,n);
  fill_random_matrix(b,n);

  tick(&tt);
  matmult_transpose(a,b,c,n);
  printf("Time to multiply %d x %d matrices with transpose multiplication is %9.4f\n",n,n,tocksilent(&tt));
  printf("Sum of elements is %14.4e\n",matrix_sum(c,n));

}
